import java.net.*;
import java.io.*;
import java.util.*;
import java.lang.*;

class ClassParser {
	private Vector constantPoolTable = new Vector();
	private DataInputStream s;
	private boolean initCalled = false;   
	//Vector InterfacesTable = new Vector();
	Vector fieldsTable = new Vector();
	Vector methodsTable = new Vector();

	ClassParser(URL data){
		try {
			s = new DataInputStream(data.openStream());
		} catch (FileNotFoundException e){
			System.err.println(e);
		}catch (IOException e){
			System.err.println(e);
		}
	}
	ClassParser(){}
	ClassParser(String s){}
	void init(){System.out.println("init has been called ....................");}
	private void isInitCalled(){
		if (!initCalled) init();
	}
	public void fillConstantPoolTable () throws IOException{
		isInitCalled();
		s.readInt();               //Magic
		s.readUnsignedShort();     //Minor_version
		s.readUnsignedShort();     //Major_version

		int constant_count = s.readUnsignedShort();
		for (int k = 1; k < constant_count; k++){
			int cp_info_tag = s.readUnsignedByte();
			ConstantPool cp = new ConstantPool();
			cp.tag = cp_info_tag;
			
			switch (cp_info_tag) {				
			case 7:     //CONSTANT_Class		
				cp.name_index = s.readUnsignedShort();
				break;				
			case 9:     //CONSTANT_Fieldref
				cp.class_index = s.readUnsignedShort();
				cp.name_and_type_index = s.readUnsignedShort();
				break;				
			case 10:    //CONSTANT_Methodref
				cp.class_index = s.readUnsignedShort();
				cp.name_and_type_index = s.readUnsignedShort();
				break;				
			case 11:     //CONSTANT_InterfaceMethodref
				cp.class_index = s.readUnsignedShort();
				cp.name_and_type_index = s.readUnsignedShort();
				break;				
			case 8:     //CONSTANT_String
				cp.string_index = s.readUnsignedShort();
				break;				
			case 3 :     //CONSTANT_Integer
				cp.bytes = s.readInt();
				break;				
			case 4:     //CONSTANT_Float
				cp.bytes = s.readInt();
				break;				
			case 5:     //CONSTANT_Long
				cp.high_bytes = s.readInt();
				cp.low_bytes = s.readInt();
				break;				
			case 6:     //CONSTANT_Double
				cp.high_bytes = s.readInt();
				cp.low_bytes = s.readInt();
				break;
			case 12:     //CONSTANT_NameAndType
				cp.name_index = s.readUnsignedShort();
				cp.descriptor_index = s.readUnsignedShort();
				break;				
			case 1:     //CONSTANT_Utf8
				int t = s.readUnsignedShort();
				String st = "";
				for (int i = 0; i < t; i++)
					st += (char)s.readUnsignedByte();
				
				cp.utf8_info = st;
				//				System.out.println("constant pool : "+st);
				break;
			}
			constantPoolTable.addElement(cp);
			if ((cp_info_tag == 5) || (cp_info_tag == 6)){
				constantPoolTable.addElement(null);
				k++;
			}
		}
		System.out.println("modifier :"+s.readUnsignedShort());     //Access_flags

		System.out.println("name of class : "+printConstantPoolTable(s.readUnsignedShort()));     //This_class
		s.readUnsignedShort();     //super_class

		int interface_count = s.readUnsignedShort();  //Interface_count
		System.out.println("interface test ******** : "+interface_count);
		for (int i = 0; i < interface_count; i++){    //Interface Info
			//s.readUnsignedShort();
			//if we need interface then use follwing with modify....
			System.out.println("interface : "+printConstantPoolTable(s.readUnsignedShort()));
		}

		int field_count = s.readUnsignedShort();     //Field_count
		System.out.println("Field count : "+field_count);
		for (int i = 0; i < field_count; i++)        //fill Fields Info
			fillFieldsTable();

		int method_count = s.readUnsignedShort();     //method_count
		System.out.println("Method count : "+method_count);
		for (int i = 0; i < method_count; i++)        //fill Methods Info
			fillMethodsTable();

		int attribute_count = s.readUnsignedShort();  //attribute_count
		System.out.println("Attribute count : "+attribute_count);
		for (int i = 0; i < attribute_count; i++)     //attribute Info
			printAttributes();
	}

	private String printConstantPoolTable(int index) {

		ConstantPool consPool = (ConstantPool)constantPoolTable.elementAt(index-1);

		switch (consPool.tag) {
		case 7:     //CONSTANT_Class	
			return printConstantPoolTable(consPool.name_index);
			//break;
		case 8:     //CONSTANT_String
			return printConstantPoolTable(consPool.string_index);
			//break;
		case 1:     //CONSTANT_Utf8
			return consPool.utf8_info;
			//break;
		default:
			System.out.println("*********************************************");
			System.out.println("***** default meet ******");
			/*
		case 9:     //CONSTANT_Fieldref
			printConstantPoolTable(consPool.class_index);
			System.out.println(consPool.name_and_type_index);
			break;
		case 10:    //CONSTANT_Methodref
			printConstantPoolTable(consPool.class_index);
			System.out.println(consPool.name_and_type_index);
			break;
		case 11:     //CONSTANT_InterfaceMethodref
			printConstantPoolTable(consPool.class_index);
			System.out.println(consPool.name_and_type_index);
			break;

		case 3 :     //CONSTANT_Integer
			System.out.println(consPool.bytes);
			break;
		case 4:     //CONSTANT_Float
			System.out.println(consPool.bytes);
			break;
		case 5:     //CONSTANT_Long
			System.out.println(consPool.high_bytes);
			System.out.println(consPool.low_bytes);
			break;
		case 6:     //CONSTANT_Double
			System.out.println(consPool.high_bytes);
			System.out.println(consPool.low_bytes);
			break;
		case 12:     //CONSTANT_NameAndType
		    printConstantPoolTable(consPool.name_index);
			printConstantPoolTable(consPool.descriptor_index);
			break;
			*/
		}
		return null;
	}

	private void fillFieldsTable () throws IOException{
		FieldMethodInfo fmi = new FieldMethodInfo();
		fmi.access_flags = s.readUnsignedShort(); 
		fmi.name = printConstantPoolTable(s.readUnsignedShort());
		//System.out.println("name of field: "+fmi.name);
		fmi.descriptor = printConstantPoolTable(s.readUnsignedShort());
		fieldsTable.addElement(fmi);

		int attribute_count = s.readUnsignedShort();
		for (int i = 0; i < attribute_count; i++)
			printAttributes();
	}

	private void fillMethodsTable () throws IOException{
		FieldMethodInfo fmi = new FieldMethodInfo();
		fmi.access_flags = s.readUnsignedShort(); 
		fmi.name = printConstantPoolTable(s.readUnsignedShort());
		System.out.println("methods name : "+fmi.name+"  flages: "+fmi.access_flags);
		fmi.descriptor = printConstantPoolTable(s.readUnsignedShort());
		methodsTable.addElement(fmi);

		int attribute_count = s.readUnsignedShort();
		for (int i = 0; i < attribute_count; i++)
			printAttributes();
	}

	private void printAttributes () throws IOException{
		String name = printConstantPoolTable(s.readUnsignedShort());
		//System.out.println("attribute test : "+name);
		/*		
		if (name.equals("InnerClasses")){
			s.readInt();    //attribute length
			int length = s.readUnsignedShort();
			System.out.println("innerClass length : "+length);
			for (int i = 0; i < length; i++){
				String innerName = printConstantPoolTable(s.readUnsignedShort());
				System.out.println("Inner_class_info ["+i+"] = "+innerName);
				innerName += ".class";

				System.out.println("outer_class_info ["+i+"] = "+printConstantPoolTable(s.readUnsignedShort()));				
				System.out.println("Inner_name ["+i+"] = "+printConstantPoolTable(s.readUnsignedShort()));				
				System.out.println("Inner_class_access_flage ["+i+"] = "+s.readUnsignedShort());
			}
			} else*/ if (name.equals("Exceptions")){
			s.readInt();
			int length = s.readUnsignedShort();
			for (int i = 0; i < length; i++)
				System.out.println("exception ["+i+"] = "+printConstantPoolTable(s.readUnsignedShort()).replace('/','.'));
		}else {
			int length = s.readInt();     //attribute_length
			for (int i = 0; i < length; i++)
				s.readUnsignedByte();
		}
	}
	test1.test2 interclasstest ;

	public interface interfaceTest{
		static final int CONSTANT2 = 4;
	}

	class c implements interfaceTest{
		void f() {
			int i = CONSTANT2;
		}
	}
	class ConstantPool {
        int tag;
        int name_index;
        int class_index;
        int name_and_type_index;
        int string_index;
        int bytes;
        int high_bytes;
        int low_bytes;
        int descriptor_index;
        String utf8_info;
	}
}


